@@ -6,6 +6,8 @@ import aerosandbox.numpy as np
 from aerosandbox.aerodynamics.aero_3D.aero_buildup_submodels.fuselage_aerodynamics_utilities import *
 from aerosandbox.library.aerodynamics import transonic
 import aerosandbox.library.aerodynamics as aerolib
+import copy
+from typing import Union, List, Dict, Any
 
 
 class AeroBuildup(ExplicitAnalysis):
@@ -25,23 +27,6 @@ class AeroBuildup(ExplicitAnalysis):
 
     """
     default_analysis_specific_options = {
-        Airplane: dict(),
-        Wing    : dict(
-            additional_CL=0,
-            additional_CD=0,
-            additional_CM=0,
-            additional_L=0,
-            additional_D=0,
-            additional_M=0,
-        ),
-        WingXSec: dict(
-            additional_CL=0,
-            additional_CD=0,
-            additional_CM=0,
-            additional_L=0,
-            additional_D=0,
-            additional_M=0,
-        ),
         Fuselage: dict(
             E_wave_drag=2.5,  # Wave drag efficiency factor
 
@@ -58,20 +43,13 @@ class AeroBuildup(ExplicitAnalysis):
             nose_fineness_ratio=3,  # Fineness ratio (length / diameter) of the nose section of the fuselage.
 
             # Impacts wave drag calculations, among other things.
-
-            additional_CL=0,
-            additional_CD=0,
-            additional_CM=0,
-            additional_L=0,
-            additional_D=0,
-            additional_M=0,
         ),
     }
 
     def __init__(self,
                  airplane: Airplane,
                  op_point: OperatingPoint,
-                 xyz_ref: List[float] = None,
+                 xyz_ref: Union[np.ndarray, List[float]] = None,
                  include_wave_drag: bool = True,
                  ):
         super().__init__()
@@ -166,18 +144,26 @@ class AeroBuildup(ExplicitAnalysis):
                 else:
                     aero_total[k] = aero_total[k] + aero_component[k]
 
-        ##### Add nondimensional forces, and nondimensional quantities.
+        ##### Compute dimensionalization factor
         if self.airplane.s_ref is not None:
             qS = self.op_point.dynamic_pressure() * self.airplane.s_ref
+            c = self.airplane.c_ref
+            b = self.airplane.b_ref
+        else:
+            raise ValueError(
+                "Airplane must have a reference area and length attributes.\n"
+                "(`Airplane.s_ref`, `Airplane.c_ref`, `airplane.b_ref`)"
+            )
 
-            aero_total["CL"] = aero_total["L"] / qS
-            aero_total["CY"] = aero_total["Y"] / qS
-            aero_total["CD"] = aero_total["D"] / qS
-            aero_total["Cl"] = aero_total["l_b"] / qS / self.airplane.b_ref
-            aero_total["Cm"] = aero_total["m_b"] / qS / self.airplane.c_ref
-            aero_total["Cn"] = aero_total["n_b"] / qS / self.airplane.b_ref
+        ##### Add nondimensional forces, and nondimensional quantities.
+        aero_total["CL"] = aero_total["L"] / qS
+        aero_total["CY"] = aero_total["Y"] / qS
+        aero_total["CD"] = aero_total["D"] / qS
+        aero_total["Cl"] = aero_total["l_b"] / qS / b
+        aero_total["Cm"] = aero_total["m_b"] / qS / c
+        aero_total["Cn"] = aero_total["n_b"] / qS / b
 
-            self.output = aero_total
+        self.output = aero_total
 
         return aero_total
 
@@ -258,6 +244,16 @@ class AeroBuildup(ExplicitAnalysis):
                         * scaling_factors[derivative_denominator]
                 )
 
+            ### Try to compute and append neutral point, if possible
+            if derivative_denominator == "alpha":
+                run_base["x_np"] = self.xyz_ref[0] - (
+                        run_base["Cma"] * (self.airplane.c_ref / run_base["CLa"])
+                )
+            if derivative_denominator == "beta":
+                run_base["x_np_lateral"] = self.xyz_ref[0] - (
+                        run_base["Cnb"] * (self.airplane.b_ref / run_base["CYb"])
+                )
+
         return run_base
 
     def wing_aerodynamics(self,
@@ -279,31 +275,57 @@ class AeroBuildup(ExplicitAnalysis):
         """
         ##### Alias a few things for convenience
         op_point = self.op_point
-        wing_options = self.get_options(wing)
+        # wing_options = self.get_options(wing) # currently no wing options
 
         ##### Compute general wing properties
-        sweep = wing.mean_sweep_angle()
-        AR = wing.aspect_ratio()
+        wing_MAC = wing.mean_aerodynamic_chord()
+        wing_taper = wing.taper_ratio()
+        wing_sweep = wing.mean_sweep_angle()
+        AR_effective = wing.aspect_ratio(type="effective")
+        AR_geometric = wing.aspect_ratio(type="geometric")
         mach = op_point.mach()
-        mach_normal = mach * np.cosd(sweep)
-        CL_over_Cl = aerolib.CL_over_Cl(
-            aspect_ratio=AR,
+        # mach_normal = mach * np.cosd(sweep)
+        AR_3D_factor = aerolib.CL_over_Cl(
+            aspect_ratio=AR_effective,
             mach=mach,
-            sweep=sweep,
+            sweep=wing_sweep,
             Cl_is_compressible=True
         )
         oswalds_efficiency = aerolib.oswalds_efficiency(
-            taper_ratio=wing.taper_ratio(),
-            aspect_ratio=AR,
-            sweep=sweep,
+            taper_ratio=wing_taper,
+            aspect_ratio=AR_effective,
+            sweep=wing_sweep,
             fuselage_diameter_to_span_ratio=0  # an assumption
         )
         areas = wing.area(_sectional=True)
+
         aerodynamic_centers = wing.aerodynamic_center(_sectional=True)
 
+        ### Model for the neutral point movement due to lifting-line unsweep near centerline
+        # See /studies/AeroBuildup_LL_unsweep_calibration
+        a = AR_effective / (AR_effective + 2)
+        s = np.radians(wing_sweep)
+        t = np.exp(-wing_taper)
+        neutral_point_deviation_due_to_unsweep = -(
+            ((((3.557726 ** (a ** 2.8443985)) * ((((s * a) + (t * 1.9149417)) + -1.4449639) * s)) + (a + -0.89228547)) * -0.16073418)
+        ) * wing_MAC
+        aerodynamic_centers = [
+            ac + np.array([neutral_point_deviation_due_to_unsweep, 0, 0])
+            for ac in aerodynamic_centers
+        ]
+
+        xsec_quarter_chords = [
+            wing._compute_xyz_of_WingXSec(
+                index=i,
+                x_nondim=0.25,
+                y_nondim=0,
+            )
+            for i in range(len(wing.xsecs))
+        ]
+
         def compute_section_aerodynamics(
                 sect_id: int,
-                mirror_across_XZ=False
+                mirror_across_XZ: bool = False
         ):
             """
             Computes the forces and moments about self.xyz_ref on a given wing section.
@@ -331,10 +353,14 @@ class AeroBuildup(ExplicitAnalysis):
             mean_chord = (xsec_a.chord + xsec_b.chord) / 2
 
             ##### Compute the local frame of this section.
-            _, _, zg_local = wing._compute_frame_of_section(sect_id)
-            zg_local = [zg_local[0], zg_local[1], zg_local[2]]
+            xg_local, yg_local, zg_local = wing._compute_frame_of_section(sect_id)
+            xg_local = [xg_local[0], xg_local[1], xg_local[2]]  # convert it to a list
+            yg_local = [yg_local[0], yg_local[1], yg_local[2]]  # convert it to a list
+            zg_local = [zg_local[0], zg_local[1], zg_local[2]]  # convert it to a list
             if mirror_across_XZ:
-                zg_local[1] *= -1
+                xg_local[1] *= -1
+                yg_local[1] *= -1
+                zg_local[1] *= -1  # Note: if mirrored, this results in a left-handed coordinate system.
 
             ##### Compute the moment arm from the section AC
             sect_AC_raw = aerodynamic_centers[sect_id]
@@ -346,8 +372,8 @@ class AeroBuildup(ExplicitAnalysis):
                 for i in range(3)
             ]
 
-            ##### Compute the generalized angle of attack, so the geometric alpha that the wing section "sees".
-            vel_vector_g_from_freestream = op_point.convert_axes(
+            ##### Compute the generalized angle of attack, which is the geometric alpha that the wing section "sees".
+            vel_vector_g_from_freestream = op_point.convert_axes(  # Points backwards (with relative wind)
                 x_from=-op_point.velocity, y_from=0, z_from=0,
                 from_axes="wind",
                 to_axes="geometry"
@@ -370,37 +396,68 @@ class AeroBuildup(ExplicitAnalysis):
                 vel_vector_g[i] / vel_mag_g
                 for i in range(3)
             ]
+            vel_dot_x = np.dot(vel_dir_g, xg_local, manual=True)
             vel_dot_z = np.dot(vel_dir_g, zg_local, manual=True)
 
-            alpha_generalized = 90 - np.arccosd(np.clip(vel_dot_z, -1, 1))
+            # alpha_generalized = 90 - np.arccosd(np.clip(vel_dot_z, -1, 1)) # In range (-90 to 90)
+            alpha_generalized = np.where(
+                vel_dot_x > 0,
+                90 - np.arccosd(np.clip(vel_dot_z, -1, 1)),  # In range (-90 to 90)
+                90 + np.arccosd(np.clip(vel_dot_z, -1, 1))  # In range (90 to 270)
+            )
 
-            # Compute the control surface deflection
-            n_surfs = len(xsec_a.control_surfaces)
-            if n_surfs == 0:
-                deflection = 0.
-            elif n_surfs == 1:
-                surf = xsec_a.control_surfaces[0]
-                deflection = surf.deflection
-                if mirror_across_XZ:
-                    if not surf.symmetric:
-                        deflection *= -1
-            else:
-                raise NotImplementedError(
-                    "AeroBuildup currently cannot handle multiple control surfaces attached to a given WingXSec.")
+            ##### Compute the effective generalized angle of attack, which roughly accounts for self-downwash
+            # effects (e.g., finite-wing effects on lift curve slope). Despite this being a tuned heuristic,
+            # it is surprisingly accurate! (<20% lift coefficient error against wind tunnel experiment, even at as
+            # low as AR = 0.5.)
+            alpha_generalized_effective = (
+                    alpha_generalized -
+                    (1 - AR_3D_factor ** 0.8) * np.sind(2 * alpha_generalized) / 2 * (180 / np.pi)
+            )  # Models finite-wing increase in alpha_{CL_max}.
 
-            # Compute Reynolds numbers
+            ##### Compute the control surface deflection
+            deflection = 0.
+            for surf in xsec_a.control_surfaces:
+                if mirror_across_XZ and not surf.symmetric:
+                    deflection -= surf.deflection
+                else:
+                    deflection += surf.deflection
+
+            ##### Compute sweep angle
+            xsec_a_quarter_chord = xsec_quarter_chords[sect_id]
+            xsec_b_quarter_chord = xsec_quarter_chords[sect_id + 1]
+            quarter_chord_vector_g = xsec_b_quarter_chord - xsec_a_quarter_chord
+            quarter_chord_dir_g = quarter_chord_vector_g / np.linalg.norm(quarter_chord_vector_g)
+            quarter_chord_dir_g = [  # Convert to list
+                quarter_chord_dir_g[0],
+                quarter_chord_dir_g[1],
+                quarter_chord_dir_g[2],
+            ]
+
+            vel_dot_quarter_chord = np.dot(
+                vel_dir_g,
+                quarter_chord_dir_g,
+                manual=True
+            )
+
+            sweep_rad = np.arcsin(vel_dot_quarter_chord)
+
+            ##### Compute Reynolds numbers
             Re_a = op_point.reynolds(xsec_a.chord)
             Re_b = op_point.reynolds(xsec_b.chord)
 
-            ##### Compute sectional lift at cross sections using lookup functions. Merge them linearly to get section CL.
+            ##### Compute Mach numbers
+            mach_normal = mach * np.cos(sweep_rad)
+
+            ##### Compute sectional lift at cross-sections using lookup functions. Merge them linearly to get section CL.
             xsec_a_args = dict(
-                alpha=alpha_generalized,
+                alpha=alpha_generalized_effective,
                 Re=Re_a,
                 mach=mach_normal,
                 deflection=deflection
             )
             xsec_b_args = dict(
-                alpha=alpha_generalized,
+                alpha=alpha_generalized_effective,
                 Re=Re_b,
                 mach=mach_normal,
                 deflection=deflection
@@ -411,14 +468,18 @@ class AeroBuildup(ExplicitAnalysis):
             sect_CL = (
                               xsec_a_Cl * a_weight +
                               xsec_b_Cl * b_weight
-                      ) * CL_over_Cl
+                      ) * AR_3D_factor ** 0.2  # Models slight decrease in finite-wing CL_max.
 
             ##### Compute sectional drag at cross sections using lookup functions. Merge them linearly to get section CD.
             xsec_a_Cdp = xsec_a.airfoil.CD_function(**xsec_a_args)
             xsec_b_Cdp = xsec_b.airfoil.CD_function(**xsec_b_args)
             sect_CDp = (
-                    xsec_a_Cdp * a_weight +
-                    xsec_b_Cdp * b_weight
+                    (
+                            xsec_a_Cdp * a_weight +
+                            xsec_b_Cdp * b_weight
+                    ) *
+                    (1 + 0.2 / AR_geometric * np.cosd(alpha_generalized_effective) ** 2)
+                # accounts for extra form factor of tip area, and 3D effects (crossflow trips)
             )
 
             ##### Compute sectional moment at cross sections using lookup functions. Merge them linearly to get section CM.
@@ -431,7 +492,7 @@ class AeroBuildup(ExplicitAnalysis):
 
             ##### Compute induced drag from local CL and full-wing properties (AR, e)
             sect_CDi = (
-                    sect_CL ** 2 / (np.pi * AR * oswalds_efficiency)
+                    sect_CL ** 2 / (np.pi * AR_effective * oswalds_efficiency)
             )
 
             ##### Total the drag.
@@ -449,6 +510,14 @@ class AeroBuildup(ExplicitAnalysis):
                 zg_local[i] - vel_dot_z * vel_dir_g[i]
                 for i in range(3)
             ]
+            L_direction_g_unnormalized = [  # Handles the 90 degree to 270 degree cases
+                np.where(
+                    vel_dot_x > 0,
+                    L_direction_g_unnormalized[i],
+                    -1 * L_direction_g_unnormalized[i],
+                )
+                for i in range(3)
+            ]
             L_direction_g_mag = np.sqrt(sum([comp ** 2 for comp in L_direction_g_unnormalized]))
             L_direction_g = [
                 L_direction_g_unnormalized[i] / L_direction_g_mag
@@ -604,9 +673,9 @@ class AeroBuildup(ExplicitAnalysis):
 
             def soft_norm(xyz):
                 return (
-                               sum([comp ** 2 for comp in xyz])
-                               + 1e-100  # Keeps the derivative from exploding
-                       ) ** 0.5
+                        sum([comp ** 2 for comp in xyz])
+                        + 1e-100  # Keeps the derivative from exploding
+                ) ** 0.5
 
             generalized_alpha = 2 * np.arctan2d(
                 soft_norm([vel_direction_g[i] - xg_local[i] for i in range(3)]),
