@@ -149,7 +149,7 @@ class _DynamicsPointMassBaseClass(AeroSandboxObject, ABC):
     def control_variables(self) -> Dict[str, Union[float, np.ndarray]]:
         pass
 
-    def __repr__(self):
+    def __repr__(self) -> str:
 
         title = f"{self.__class__.__name__} instance:"
 
@@ -183,10 +183,10 @@ class _DynamicsPointMassBaseClass(AeroSandboxObject, ABC):
             control_variables
         ])
 
-    def __getitem__(self, index):
+    def __getitem__(self, index: int):
         """
         Indexes one item from each attribute of a Dynamics instance.
-        Returns a new Dynamics instance.
+        Returns a new Dynamics instance of the same type.
 
         Args:
             index: The index that is being called; e.g.,:
@@ -239,7 +239,8 @@ class _DynamicsPointMassBaseClass(AeroSandboxObject, ABC):
                               opti: Opti,
                               time: np.ndarray,
                               method: str = "midpoint",
-                              which: Union[str, List[str], Tuple[str]] = "all"
+                              which: Union[str, List[str], Tuple[str]] = "all",
+                              _stacklevel=1,
                               ):
         """
         Applies the relevant state derivative constraints to a given Opti instance.
@@ -262,6 +263,11 @@ class _DynamicsPointMassBaseClass(AeroSandboxObject, ABC):
                     * A list of strings that are state variable names (i.e., a subset of `dyn.state.keys()`),
                     that gives the names of state variables to be constrained.
 
+            _stacklevel: Optional and advanced, purely used for debugging. Allows users to correctly track where
+            constraints are declared in the event that they are subclassing `aerosandbox.Opti`. Modifies the
+            stacklevel of the declaration tracked, which is then presented using
+            `aerosandbox.Opti.variable_declaration()` and `aerosandbox.Opti.constraint_declaration()`.
+
         Returns:
 
         """
@@ -283,6 +289,7 @@ class _DynamicsPointMassBaseClass(AeroSandboxObject, ABC):
                     variable=self.state[state_var_name],
                     with_respect_to=time,
                     method=method,
+                    _stacklevel=_stacklevel + 1
                 )
             except KeyError:
                 raise ValueError(f"This dynamics instance does not have a state named '{state_var_name}'!")
@@ -325,9 +332,9 @@ class _DynamicsPointMassBaseClass(AeroSandboxObject, ABC):
 
     @abstractmethod
     def add_force(self,
-                  Fx: Union[np.ndarray, float] = 0,
-                  Fy: Union[np.ndarray, float] = 0,
-                  Fz: Union[np.ndarray, float] = 0,
+                  Fx: Union[float, np.ndarray] = 0,
+                  Fy: Union[float, np.ndarray] = 0,
+                  Fz: Union[float, np.ndarray] = 0,
                   axes: str = "wind",
                   ) -> None:
         """
