@@ -8,7 +8,6 @@ from aerosandbox.geometry.airfoil.default_airfoil_aerodynamics import default_CL
 from aerosandbox.library.aerodynamics import transonic
 from aerosandbox.modeling.splines.hermite import linear_hermite_patch, cubic_hermite_patch
 from scipy import interpolate
-import matplotlib.pyplot as plt
 from typing import Callable, Union, Any, Dict
 import json
 from pathlib import Path
@@ -160,9 +159,11 @@ class Airfoil(Polygon):
                         include_compressibility_effects: bool = True,
                         transonic_buffet_lift_knockdown: float = 0.3,
                         make_symmetric_polars: bool = False,
+                        add_deflections_as_all_moving_surfaces: bool = True,
                         ) -> None:
         """
-        Generates airfoil polars (CL, CD, CM functions) and assigns them in-place to this Airfoil's polar functions.
+        Generates airfoil polar surrogate models (CL, CD, CM functions) from XFoil data and assigns them in-place to
+        this Airfoil's polar functions.
 
         In other words, when this function is run, the following functions will be added (or overwritten) to the instance:
             * Airfoil.CL_function(alpha, Re, mach, deflection)
@@ -180,6 +181,10 @@ class Airfoil(Polygon):
             cache_filename: A path-like filename (ideally a "*.json" file) that can be used to cache the XFoil
             results, making it much faster to regenerate the results.
 
+                * If the file does not exist, XFoil will be run, and a cache file will be created.
+
+                * If the file does exist, XFoil will not be run, and the cache file will be read instead.
+
             xfoil_kwargs: Keyword arguments to pass into the AeroSandbox XFoil module. See the aerosandbox.XFoil
             constructor for options.
 
@@ -191,6 +196,11 @@ class Airfoil(Polygon):
             mach tuck, CL effects across normal shocks. Note that accuracy here is dubious in the transonic regime
             and above - you should really specify your own CL/CD/CM models
 
+            add_deflections_as_all_moving_surfaces: If this boolean is flagged `True`, the `deflection=` keyword
+            argument of the generated polar functions directly modifies the angle of attack. This effectively means
+            that you are assuming an all-moving airfoil hinged about the quarter-chord. If this boolean is flagged
+            `False`, the `deflection=` keyword argument will do nothing.
+
         Warning: In-place operation! Modifies this Airfoil object by setting Airfoil.CL_function, etc. to the new
         polars.
 
@@ -206,7 +216,7 @@ class Airfoil(Polygon):
         if unstructured_interpolated_model_kwargs is None:
             unstructured_interpolated_model_kwargs = {}
 
-        xfoil_kwargs = {  # See asb.XFoil for documentation on these.
+        xfoil_kwargs = {  # See asb.XFoil for the documentation on these.
             "verbose"      : False,
             "max_iter"     : 20,
             "xfoil_repanel": True,
@@ -225,7 +235,7 @@ class Airfoil(Polygon):
             **unstructured_interpolated_model_kwargs
         }
 
-        ### Retrieve XFoil Polar Data from cache, if it exists.
+        ### Retrieve XFoil Polar Data from the cache, if it exists.
         data = None
         if cache_filename is not None:
             try:
@@ -268,7 +278,7 @@ class Airfoil(Polygon):
             }
 
             if make_symmetric_polars:  # If the airfoil is known to be symmetric, duplicate all data across alpha.
-                keys_symmetric_across_alpha = ['CD', 'CDp', 'Re']
+                keys_symmetric_across_alpha = ['CD', 'CDp', 'Re']  # Assumes the rest are antisymmetric
 
                 data = {
                     k: np.concatenate([v, v if k in keys_symmetric_across_alpha else -v])
@@ -369,14 +379,17 @@ class Airfoil(Polygon):
         )
 
         def CL_function(alpha, Re, mach=0, deflection=0):
+            if add_deflections_as_all_moving_surfaces:
+                alpha = alpha + deflection
+
             alpha = np.mod(alpha + 180, 360) - 180  # Keep alpha in the valid range.
             CL_attached = CL_attached_interpolator({
                 "alpha": alpha,
                 "ln_Re": np.log(Re),
             })
-            CL_separated = CL_separated_interpolator(alpha)
+            CL_separated = CL_separated_interpolator(alpha)  # Lift coefficient if separated
 
-            CL_mach_0 = np.blend(
+            CL_mach_0 = np.blend(  # Lift coefficient at mach = 0
                 separation_parameter(alpha, Re),
                 CL_separated,
                 CL_attached
@@ -421,6 +434,9 @@ class Airfoil(Polygon):
                 return CL_mach_0
 
         def CD_function(alpha, Re, mach=0, deflection=0):
+            if add_deflections_as_all_moving_surfaces:
+                alpha = alpha + deflection
+
             alpha = np.mod(alpha + 180, 360) - 180  # Keep alpha in the valid range.
             log10_CD_attached = log10_CD_attached_interpolator({
                 "alpha": alpha,
@@ -518,6 +534,9 @@ class Airfoil(Polygon):
                 return 10 ** log10_CD_mach_0
 
         def CM_function(alpha, Re, mach=0, deflection=0):
+            if add_deflections_as_all_moving_surfaces:
+                alpha = alpha + deflection
+
             alpha = np.mod(alpha + 180, 360) - 180  # Keep alpha in the valid range.
             CM_attached = CM_attached_interpolator({
                 "alpha": alpha,
@@ -643,6 +662,7 @@ class Airfoil(Polygon):
             y_mcl = self.local_camber(x_mcl)
 
         if backend == "matplotlib":
+            import matplotlib.pyplot as plt
             color = '#280887'
             plt.plot(x, y, ".-", zorder=11, color=color)
             plt.fill(x, y, zorder=10, color=color, alpha=0.2)
@@ -801,7 +821,10 @@ class Airfoil(Polygon):
 
         return Airfoil(
             name=self.name,
-            coordinates=stack_coordinates(x, y)
+            coordinates=stack_coordinates(x, y),
+            CL_function=self.CL_function,
+            CD_function=self.CD_function,
+            CM_function=self.CM_function,
         )
 
     def add_control_surface(
