@@ -14,14 +14,15 @@ class Wing(AeroSandboxObject):
 
     Anatomy of a Wing:
 
-        A wing consists chiefly of a collection of cross sections, or "xsecs". These can be accessed with `Wing.xsecs`,
-        which gives a list of xsecs in the Wing. Each xsec is a WingXSec object, a class that is defined separately.
+        A wing consists chiefly of a collection of cross sections, or "xsecs". A cross section is a 2D "slice" of a
+        wing. These can be accessed with `Wing.xsecs`, which gives a list of xsecs in the Wing. Each xsec is a
+        WingXSec object, a class that is defined separately.
 
         You may also see references to wing "sections", which are different than cross sections (xsecs)! Sections are
         the portions of the wing that are in between xsecs. In other words, a wing with N cross sections (xsecs,
         WingXSec objects) will always have N-1 sections. Sections are never explicitly defined, since you can get all
-        needed information by lofting from the adjacent cross sections. For example, section 0 is a loft between
-        cross sections 0 and 1.
+        needed information by lofting from the adjacent cross sections. For example, section 0 (the first one) is a
+        loft between cross sections 0 and 1.
 
         Wings are lofted linearly between cross sections.
 
@@ -32,7 +33,7 @@ class Wing(AeroSandboxObject):
     """
 
     def __init__(self,
-                 name: str = None,
+                 name: Optional[str] = None,
                  xsecs: List['WingXSec'] = None,
                  symmetric: bool = False,
                  analysis_specific_options: Optional[Dict[type, Dict[str, Any]]] = None,
@@ -45,7 +46,7 @@ class Wing(AeroSandboxObject):
 
             name: Name of the wing [optional]. It can help when debugging to give each wing a sensible name.
 
-            xsecs: A list of wing cross ("X") sections in the form of WingXSec objects.
+            xsecs: A list of wing cross sections ("xsecs") in the form of WingXSec objects.
 
             symmetric: Is the wing symmetric across the XZ plane?
 
@@ -129,11 +130,19 @@ class Wing(AeroSandboxObject):
              ) -> float:
         """
         Returns the span, with options for various ways of measuring this.
+
          * wetted: Adds up YZ-distances of each section piece by piece
+
          * y: Adds up the Y-distances of each section piece by piece
+
          * z: Adds up the Z-distances of each section piece by piece
+
          * y-full: Y-distance between the XZ plane and the tip of the wing. (Can't be used with _sectional).
-        If symmetric, this is doubled left/right to obtain the full span.
+
+        If symmetric, this is doubled left/right to obtain the full span. With symmetric wings, note that if the root
+        cross-section is not coincident with the centerline, the fictitious span connecting the left and right root
+        cross-sections is not included. The only exception to this is the "y-full" option, which computes the span to
+        the XZ plane (hence, including the center part).
 
         Args:
             type: One of the above options, as a string.
@@ -199,12 +208,17 @@ class Wing(AeroSandboxObject):
     def area(self,
              type: str = "wetted",
              _sectional: bool = False,
-             ) -> float:
+             ) -> Union[float, List[float]]:
         """
-        Returns the area, with options for various ways of measuring this.
+        Computes the wing area, with options for various ways of measuring this.
+
          * wetted: wetted area
+
          * projected: area projected onto the XY plane (top-down view)
-        If symmetric, this is doubled left/right to obtain the full wing area.
+
+        If symmetric, this area is doubled left/right to obtain the full wing area. With symmetric wings, note that if the
+        root cross-section is not coincident with the centerline, the fictitious area connecting the left and right
+        root cross-sections is not included.
 
         Args:
             type: One of the above options, as a string.
@@ -245,10 +259,33 @@ class Wing(AeroSandboxObject):
 
         return area
 
-    def aspect_ratio(self) -> float:
-        # Returns the aspect ratio (b^2/S).
-        # Uses the full span and the full area if symmetric.
-        return self.span() ** 2 / self.area()
+    def aspect_ratio(self,
+                     type: str = "geometric",
+                     ) -> float:
+        """
+        Computes the aspect ratio of the wing, with options for various ways of measuring this.
+
+         * geometric: geometric aspect ratio, computed in the typical fashion (b^2 / S).
+
+         * effective: Differs from the geometric aspect ratio only in the case of symmetric wings whose root
+         cross-section is not on the centerline. In these cases, it counts 50% of the fictitious span as wing span
+
+        Args:
+            type: One of the above options, as a string.
+
+        """
+        if type == "geometric":
+            return self.span() ** 2 / self.area()
+
+        elif type == "effective":
+            effective_span = self.span() + 0.5 * self.xsecs[0].xyz_le[1]
+
+            effective_area = self.area() + 0.5 * self.xsecs[0].xyz_le[1] * self.xsecs[0].chord
+
+            return effective_span ** 2 / effective_area
+
+        else:
+            raise ValueError("Bad value of `type`!")
 
     def is_entirely_symmetric(self) -> bool:
         # Returns a boolean of whether the wing is totally symmetric (i.e.), every xsec has symmetric control surfaces.
@@ -464,8 +501,8 @@ class Wing(AeroSandboxObject):
 
         airfoil_nondim_coordinates = np.array([
             xsec.airfoil
-                .repanel(n_points_per_side=chordwise_resolution + 1)
-                .coordinates
+            .repanel(n_points_per_side=chordwise_resolution + 1)
+            .coordinates
             for xsec in self.xsecs
         ])
 
@@ -1116,7 +1153,7 @@ class WingXSec(AeroSandboxObject):
             )
 
     def __repr__(self) -> str:
-        return f"WingXSec (Airfoil: {self.airfoil.name}, chord: {self.chord:.3f}, twist: {self.twist:.3f})"
+        return f"WingXSec (Airfoil: {self.airfoil.name}, chord: {self.chord}, twist: {self.twist})"
 
     def translate(self,
                   xyz: Union[np.ndarray, List]
