@@ -219,6 +219,7 @@ def solar_flux(
         panel_azimuth_angle: Union[float, np.ndarray] = 0.,
         panel_tilt_angle: Union[float, np.ndarray] = 0.,
         air_quality: str = 'typical',
+        albedo: Union[float, np.ndarray] = 0.2,
         **deprecated_kwargs
 ) -> Union[float, np.ndarray]:
     """
@@ -258,6 +259,10 @@ def solar_flux(
 
             * A horizontal panel has a tilt angle of 0, and a vertical panel has a tilt angle of 90 degrees.
 
+            If the angle between the panel normal and the sun direction is ever more than 90 degrees (e.g. the panel
+            is pointed the wrong way), we assume that the panel receives no direct irradiation. (However,
+            it may still receive minor amounts of power due to diffuse irradiation from re-scattering.)
+
         air_quality: Indicates the amount of pollution in the air. A string, one of:
 
             * 'typical': Corresponds to "rural aerosol loading" following ASTM G-173.
@@ -266,6 +271,27 @@ def solar_flux(
 
             * 'polluted': Urban atmosphere conditions.
 
+        albedo: The fraction of light that hits the ground that is reflected. Affects illumination from re-scattering
+        when panels are tilted. Typical values for general terrestrial surfaces are 0.2, which is the default here.
+
+            * Other values, taken from the Earthscan source (citation above):
+
+                * Grass (July, August): 0.25
+                * Lawn: 0.18 - 0.23
+                * Dry grass: 0.28 - 0.32
+                * Milled fields: 0.26
+                * Barren soil: 0.17
+                * Gravel: 0.18
+                * Clean concrete: 0.30
+                * Eroded concrete: 0.20
+                * Clean cement: 0.55
+                * Asphalt: 0.15
+                * Forests: 0.05 - 0.18
+                * Sandy areas: 0.10 - 0.25
+                * Water: Strongly dependent on incidence angle; 0.05 - 0.22
+                * Fresh snow: 0.80 - 0.90
+                * Old snow: 0.45 - 0.70
+
     Returns: The solar power flux [W/m^2] on the panel.
 
         * Note: does not account for any potential reflectivity of the solar panel's coating itself.
@@ -304,7 +330,16 @@ def solar_flux(
     # Indicates that absorption and scattering happen in a 18:10 ratio, at least in AM1.5 conditions. We extrapolate
     # this to all conditions.
 
-    diffuse_irradiance = scattering_losses * atmospheric_transmission_fraction
+    panel_tilt_angle = np.mod(panel_tilt_angle, 360)
+    fraction_of_panel_facing_sky = np.where(
+        panel_tilt_angle < 180,
+        1 - panel_tilt_angle / 180,
+        -1 + panel_tilt_angle / 180,
+    )
+
+    diffuse_irradiance = scattering_losses * atmospheric_transmission_fraction * (
+            fraction_of_panel_facing_sky + albedo * (1 - fraction_of_panel_facing_sky)
+    )
     # We assume that the in-scattering (i.e., diffuse irradiance) and the out-scattering (i.e., scattering losses in
     # the direct irradiance calculation) are equal, by argument of approximately parallel incident rays.
     # We further assume that any in-scattering must then once-again go through the absorption / re-scattering process,
@@ -319,7 +354,7 @@ def solar_flux(
     cosine_of_angle_between_panel_normal_and_sun = np.fmax(
         cosine_of_angle_between_panel_normal_and_sun,
         0
-    ) # Accounts for if you have a downwards-pointing panel while the sun is above you.
+    )  # Accounts for if you have a downwards-pointing panel while the sun is above you.
 
     # Source: https://www.pveducation.org/pvcdrom/properties-of-sunlight/arbitrary-orientation-and-tilt
     # Author of this code (Peter Sharpe) has manually verified correctness of this vector math.
@@ -408,7 +443,7 @@ if __name__ == "__main__":
     import matplotlib.pyplot as plt
     import aerosandbox.tools.pretty_plots as p
 
-    plt.switch_backend('WebAgg')
+    # plt.switch_backend('WebAgg')
 
     base_color = p.palettes['categorical'][0]
     quality_colors = {
@@ -429,7 +464,7 @@ if __name__ == "__main__":
 
     fig, ax = plt.subplots(2, 1, figsize=(7, 6.5))
     plt.sca(ax[0])
-    plt.title(f"Solar Flux on a Horizontal Surface Over A Day\n(Tropic of Cancer, Summer Solstice)")
+    plt.title(f"Solar Flux on a Horizontal Surface Over A Day\n(Tropic of Cancer, Summer Solstice, Sea Level)")
     for q in quality_colors.keys():
         plt.plot(
             hour,
@@ -442,7 +477,7 @@ if __name__ == "__main__":
         )
 
     plt.sca(ax[1])
-    plt.title(f"Solar Flux on a Sun-Tracking Surface Over A Day\n(Tropic of Cancer, Summer Solstice)")
+    plt.title(f"Solar Flux on a Sun-Tracking Surface Over A Day\n(Tropic of Cancer, Summer Solstice, Sea Level)")
     for q in quality_colors.keys():
         plt.plot(
             hour,
