@@ -126,7 +126,7 @@ def interpn(
         xi: _onp.ndarray,
         method: str = "linear",
         bounds_error=True,
-        fill_value=_onp.NaN
+        fill_value=_onp.nan
 ) -> _onp.ndarray:
     """
     Performs multidimensional interpolation on regular grids. Analogue to scipy.interpolate.interpn().
@@ -135,11 +135,12 @@ def interpn(
 
     Args:
 
-        points: The points defining the regular grid in n dimensions. Tuple of coordinates of each axis.
+        points: The points defining the regular grid in n dimensions. Tuple of coordinates of each axis. Shapes (m1,
+        ), ..., (mn,)
 
-        values: The data on the regular grid in n dimensions.
+        values: The data on the regular grid in n dimensions. Shape (m1, ..., mn)
 
-        xi: The coordinates to sample the gridded data at.
+        xi: The coordinates to sample the gridded data at. (..., ndim)
 
         method: The method of interpolation to perform. one of:
 
