@@ -3,13 +3,10 @@ A set of tools used for making prettier Matplotlib plots.
 """
 
 ### General imports
-import matplotlib.pyplot as plt
 import seaborn as sns
-import matplotlib as mpl
 
 ### Local imports
-from .plotting import *
-from .plotting_with_uncertainty import *
+from .plots import *
 from .formatting import *
 from .colors import *
 from .annotation import *
