@@ -3,7 +3,7 @@ This script generates data used to estimate spar mass as a function of lift forc
 """
 
 ### Imports
-from aerosandbox.structures.beams import *
+from aerosandbox.structures.legacy.beams import *
 import scipy.io as sio
 import copy
 
